@@ -2,7 +2,7 @@
  *    Completion.java
  *    Copyright (C) 1999, 2001 Rodrigo Reyes (reyes@chez.com)
  *
- *    $Revision: 1.2 $
+ *    $Revision: 1.3 $
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -73,12 +73,35 @@ public class Completion {
     {
       case 0: return Modifier.isProtected( member.getModifiers() );
 
-      case 1: return Modifier.isPrivate( member.getModifiers() );
+      case 1: return ((Modifier.isPrivate(member.getModifiers())) ||
+                      Modifier.isProtected(member.getModifiers()));
 
       default: return false;
     }//switch
   }//met
 
+  
+  /**
+   *Returns true if the Class is accessible to the specified level of protection.
+   *@param class the Class to check
+   *@param level the level of protection. 0 means accessible to a class that 
+   *inherits from the class whose class is a member. 
+   *1 means accessible as a private only field.
+   *@return if the Class is accessible to the specified level of protection.
+   */
+  private static boolean isAccessible( Class cls, int level)
+  {
+    switch(level)
+    {
+      case 0: return Modifier.isProtected( cls.getModifiers() );
+
+      case 1: return ((Modifier.isPrivate(cls.getModifiers())) ||
+                      Modifier.isProtected(cls.getModifiers()));
+
+      default: return false;
+    }//switch
+  }//met
+  
   private static void listClassInfo(Class c, int level) 
   {
     System.out.println("(list ");
@@ -87,6 +110,7 @@ public class Completion {
     System.out.print("(list ");
     //We first get all the public fields
     Field[] fields = c.getFields();
+    
     for (int index=0; index<fields.length; index++) {
       Field field = fields[index];
     	 System.out.print("(list \""+field.getName()+"\" \""+
@@ -153,6 +177,20 @@ public class Completion {
     }
     System.out.println(")");
 	
+    //Adding inner classes
+    Class[] classes = c.getDeclaredClasses();
+    System.out.println("(list ");
+    int length = classes.length;
+    for (int i = 0; i < length; i++) {
+      Class cls = classes[i];
+      if (isAccessible(cls, level)) {
+        System.out.print("(list \"");
+        System.out.print(cls.getName() + "\"");
+        System.out.print(")");
+      } // end of if (Modifier.isPublic)
+    } // end of for (int i = 0; i < length; i++)
+    
+    System.out.println(")");
     System.out.println(")");
     return;
   }
@@ -202,7 +240,21 @@ public class Completion {
       }
       }
     System.out.println(")");
-	
+
+    //Adding inner classes
+    Class[] classes = c.getDeclaredClasses();
+    System.out.println("(list ");
+    int length = classes.length;
+    for (int i = 0; i < length; i++) {
+      Class cls = classes[i];
+      if (Modifier.isPublic(cls.getModifiers())) {
+        System.out.print("(list \"");
+        System.out.print(cls.getName() + "\"");
+        System.out.print(")");
+      } // end of if (Modifier.isPublic)
+    } // end of for (int i = 0; i < length; i++)
+    
+    System.out.println(")");
     System.out.println(")");
     return;
   }
@@ -210,8 +262,8 @@ public class Completion {
   public static void getClassInfo(String className)
   {
     try {
-	    
-      Class c = Class.forName(className);
+      DynamicClassLoader dcl = new DynamicClassLoader();
+      Class c = dcl.loadClass(className);
       if (c != null)
 	listClassInfo(c);
     } catch (ClassNotFoundException cnfe) {System.out.println("nil");}
@@ -221,7 +273,8 @@ public class Completion {
   {
     try {
 	    
-      Class c = Class.forName(className);
+      DynamicClassLoader dcl = new DynamicClassLoader();
+      Class c = dcl.loadClass(className);
       if (c != null)
 	listClassInfo(c,accessLevel);
     } catch (ClassNotFoundException cnfe) {System.out.println("nil");}
@@ -275,8 +328,24 @@ public class Completion {
 
 /*
  * $Log: Completion.java,v $
- * Revision 1.2  2001/08/16 03:29:10  andyp
- * Update to JDE 2.2.7.1
+ * Revision 1.3  2001/10/14 22:20:27  andyp
+ * Sync to 2.2.8
+ *
+ * Revision 1.10  2001/08/14 05:15:02  paulk
+ * Miscellaneous updates.
+ *
+ * Revision 1.9  2001/07/21 03:56:23  paulk
+ * Addional inner class support. Thanks to Javier Lopez.
+ *
+ * Revision 1.8  2001/07/21 03:42:24  paulk
+ * Now includes inner classes in the class info list. Thanks to Javier Lopez.
+ *
+ * Revision 1.7  2001/07/06 02:00:09  paulk
+ * Makefile
+ *
+ * Revision 1.6  2001/06/27 03:09:04  paulk
+ * The isAccessible method now returns true for both private and protected methods
+ * when the access modifier is private. Thanks to "Javier Lopez" <jlopez@cellexchange.com>.
  *
  * Revision 1.5  2000/08/10 08:46:25  paulk
  * Now outputs nil if class info not found.
